'''
Created on 2013-04-29

@author: Jackie

Task 1 of the EACL paper: relation classification.
'''

import os, glob
import numpy as np
from utilities.io import read_corenlp_doc, read_list
from sklearn.metrics import auc_score, roc_curve, auc

POSITIVE, NEGATIVE = 1, 0
START, END = 0, 1


def compute_auc(fname):
    '''
    Computes the AUC for each of the sets of scores read from the file fname.
    '''
    aucs = []
    data = read_task1_details(fname)
    for k, (labels, scores) in data.iteritems():
        val = auc_score(labels, scores)
        aucs.append(val)
        print '%s\t%.4f' % (k, val)
    print 'Mean\t%.4f' % np.mean(aucs)
    
def read_task1_details(fname):
    data = {}
    lines = open(fname).read().split('\n')
    if lines[-1] == '': lines = lines[:-1]
    
    for i in xrange(0, len(lines), 3):
        data[lines[i]] = (eval(lines[i+1]), eval(lines[i+2]))
    return data


################################################################################
# I/O and preprocessing code.
################################################################################

def read_preprocessed_items(raw_f, xml_f):
    '''Reads the preprocessed items for the relation classification task.'''
    p1, p2, items = read_raw_file(raw_f)
    doc = read_corenlp_doc(xml_f, 0)
    assert len(doc.sents) == len(items)
    for item, sent_id in zip(items, range(1, len(items) + 1)):
        item.sent = doc.sents[sent_id]
    return (p1, p2, items)


def extract_text(raw_d, raw_flist, preprocessed_d):
    for f in raw_flist:
        p1, p2, items = read_raw_file(os.path.join(raw_d, f))
        out_f = os.path.join(preprocessed_d, f)
        fh = open(out_f, 'w')
        for item in items:
            fh.write('%s\n' % item.text())
        fh.close()

def read_raw_file(filename):
    '''Reads a file in the format of Bunescu and Mooney's data set,
    and returns it as a list of ClassificationItems'''
    
    fh = open(filename, 'r')
    # omit first line: 
    fh.readline()
    p1 = fh.readline().strip().replace('_', ' ')
    p2 = fh.readline().strip().replace('_', ' ')
    items = []
    for line in fh:
        item = parse_item(line)
        if item is not None and item not in items:
            items.append(item)
    return (p1, p2, items)

def parse_item(line):
    '''Parse a line in Bunescu and Mooney's data set into a ClassificationItem,
    or None if it is not a valid entry.'''
    answer, sentence = line.strip().split('#', 1)
    try:
        answer = int(answer)
    except ValueError:
        # this case to be removed
        return None
    if answer not in [POSITIVE, NEGATIVE]:
        return None
    
    tokens = sentence.split()
    item_tokens = []
    p1_span = [-1, -1]
    p2_span = [-1, -1]
    head_span = [-1, -1]
    for token in tokens:
        if token == '<p1>':
            p1_span[START] = len(item_tokens)
        elif token == '</p1>':
            p1_span[END] = len(item_tokens)
        elif token == '<p2>':
            p2_span[START] = len(item_tokens)
        elif token == '</p2>':
            p2_span[END] = len(item_tokens)
        elif token == '<head>':
            head_span[START] = len(item_tokens)
        elif token == '</head>':
            head_span[END] = len(item_tokens)
        elif token == '__':
            item_tokens.append('UNK')
        else:
            item_tokens.append(token)
    if -1 in p1_span or -1 in p2_span:
        print 'WARNING: Unexpected span format: %s' % ' '.join(tokens)
        
    return ClassificationItem(answer, item_tokens, tuple(head_span), tuple(p1_span), tuple(p2_span))


class ClassificationItem:
    def __init__(self, label, tokens, head_span, p1_span, p2_span, sent = None):
        self.label = label # expected: POSITIVE or NEGATIVE
        self.tokens = tokens
        self.head_span = head_span # expected: tuple from start (inclusive):end (exclusive)
        self.p1_span = p1_span 
        self.p2_span = p2_span
        self.sent = sent # this is the Sentence that comes from the preprocessing
    def text(self):
        '''Returns the natural, readable text version.'''
        sent = ' '.join(self.tokens)
        sent = sent.replace(' ,' , ',')
        sent = sent.replace(' .' , '.')
        sent = sent.replace(' !' , '!')
        sent = sent.replace(' ?' , '?')
        sent = sent.replace('$ ' , '$')
        sent = sent.replace(' :' , ':')
        sent = sent.replace(' ;' , ';')
        sent = sent.replace(' %' , '%')
        sent = sent.replace(' \' s' , '\'s')
        sent = sent.replace(' \' m' , '\'m')
        sent = sent.replace(' \' ve' , '\'ve')
        sent = sent.replace(' \' re' , '\'re')
        sent = sent.replace(' \' d' , '\'d')
        sent = sent.replace(' \' ll' , '\'ll')
        sent = sent.replace(' \' ', '\' ')
        sent = sent.replace('`` ' , '"')
        sent = sent.replace(' \'\'' , '"')
        sent = sent.replace('n\' t', 'n\'t')
        sent = sent.replace('` ' , '\'')
        sent = sent.replace(' can not ' , ' cannot ')
        sent = sent.replace('( ', '(')
        sent = sent.replace(' )', ')')
        sent = sent.replace('{ ', '{')
        sent = sent.replace(' }', '}')
        sent = sent.replace('[ ', '[')
        sent = sent.replace(' ]', ']')
        sent = sent.replace(' - ', '-')
        sent = sent.replace(', "', ',"')
        sent = sent.replace('. "', '."')
        sent = sent.replace('! "', '!"')
        sent = sent.replace('? "', '?"')
        sent = sent.replace(' " ', ' "')
        if sent.startswith('" '):
            sent = '"' + sent[2:]
        return sent
    